Package: prophet
Title: Automatic Forecasting Procedure
Version: 1.0
Date: 2021-03-08
Authors@R: c(
  person("Sean", "Taylor", email = "sjtz@pm.me", role = c("cre", "aut")),
  person("Ben", "Letham", email = "bletham@fb.com", role = "aut")
  )
Description: Implements a procedure for forecasting time series data based on
    an additive model where non-linear trends are fit with yearly, weekly, and
    daily seasonality, plus holiday effects. It works best with time series
    that have strong seasonal effects and several seasons of historical data.
    Prophet is robust to missing data and shifts in the trend, and typically
    handles outliers well.
URL: https://github.com/facebook/prophet
BugReports: https://github.com/facebook/prophet/issues
Depends: R (>= 3.4.0), Rcpp (>= 0.12.0), rlang (>= 0.3.0.1)
Imports: dplyr (>= 0.7.7), dygraphs (>= 1.1.1.4), extraDistr, ggplot2,
        grid, lubridate, methods, RcppParallel (>= 5.0.1), rstan (>=
        2.18.1), rstantools (>= 2.0.0), scales, StanHeaders, stats,
        tidyr (>= 0.6.1), xts
Suggests: knitr, testthat, readr, rmarkdown
SystemRequirements: GNU make, C++11
Biarch: true
License: MIT + file LICENSE
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppParallel (>= 5.0.1),
        RcppEigen (>= 0.3.3.3.0), rstan (>= 2.18.1), StanHeaders (>=
        2.18.0)
LazyData: true
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.1.1
NeedsCompilation: yes
Packaged: 2021-03-30 00:55:01 UTC; seant
Author: Sean Taylor [cre, aut],
  Ben Letham [aut]
Maintainer: Sean Taylor <sjtz@pm.me>
Repository: CRAN
Date/Publication: 2021-03-30 12:10:07 UTC
Built: R 4.4.1; aarch64-apple-darwin21.6.0; 2024-06-16 10:27:45 UTC; unix
