# Generated by roxygen2: do not edit by hand

S3method(plot,prophet)
S3method(predict,prophet)
export(add_changepoints_to_plot)
export(add_country_holidays)
export(add_regressor)
export(add_seasonality)
export(cross_validation)
export(dyplot.prophet)
export(fit.prophet)
export(make_future_dataframe)
export(performance_metrics)
export(plot_cross_validation_metric)
export(plot_forecast_component)
export(predictive_samples)
export(prophet)
export(prophet_plot_components)
export(regressor_coefficients)
import(Rcpp)
import(RcppParallel, except = LdFlags)
import(rlang)
importFrom(dplyr,"%>%")
useDynLib(prophet, .registration = TRUE)
